APIYNFLAG_YES = "Y"
APIYNFLAG_NO = "N"
APIPASSWORD_TRADE = "T"
APIPASSWORD_PHONE = "P"
APILOGLEVEL_NONE = "N"
APILOGLEVEL_ERROR = "E"
APILOGLEVEL_WARNING = "W"
APILOGLEVEL_DEBUG = "D"
TAPI_OPTION_TYPE_AMERICAN = "A"
TAPI_OPTION_TYPE_AMERICAN2 = "B"
TAPI_OPTION_TYPE_EUROPEAN = "E"
TAPI_OPTION_TYPE_EUROPEAN2 = "F"
TAPI_COMMODITY_TYPE_NONE = "N"
TAPI_COMMODITY_TYPE_FUTURES = "F"
TAPI_COMMODITY_TYPE_OPTION = "O"
TAPI_COMMODITY_TYPE_SPREAD_MONTH = "S"
TAPI_COMMODITY_TYPE_SPREAD_COMMODITY = "M"
TAPI_COMMODITY_TYPE_BUL = "U"
TAPI_COMMODITY_TYPE_BER = "E"
TAPI_COMMODITY_TYPE_STD = "D"
TAPI_COMMODITY_TYPE_STG = "G"
TAPI_COMMODITY_TYPE_PRT = "R"
TAPI_COMMODITY_TYPE_DIRECTFOREX = "X"
TAPI_COMMODITY_TYPE_INDIRECTFOREX = "I"
TAPI_COMMODITY_TYPE_CROSSFOREX = "C"
TAPI_COMMODITY_TYPE_INDEX = "Z"
TAPI_COMMODITY_TYPE_STOCK = "T"
TAPI_COMMODITY_TYPE_TAS = "B"
TAPI_COMMODITY_TYPE_DERIVATIVE = "0"
TAPI_COMMODITY_TYPE_BULLBEAR = "1"
TAPI_COMMODITY_TYPE_INLINE = "2"
TAPI_COMMODITY_TYPE_DBET = "3"
TAPI_COMMODITY_TYPE_FUND = "4"
TAPI_COMMODITY_TYPE_MULTI_LEG = "5"
TAPI_CALLPUT_FLAG_CALL = "C"
TAPI_CALLPUT_FLAG_PUT = "P"
TAPI_CALLPUT_FLAG_NONE = "N"

TAPI_ACCOUNT_TYPE_PERSON = "P"
TAPI_ACCOUNT_TYPE_ORGANIZATION = "O"
TAPI_ACCOUNT_TYPE_AGENT = "A"
TAPI_ACCOUNT_TYPE_MARGIN = "M"
TAPI_ACCOUNT_TYPE_HOUSE = "H"
TAPI_ACCOUNT_TYPE_STOCK = "S"
TAPI_USERTYPE_CLIENT = 10000
TAPI_USERTYPE_BROKER = 20000
TAPI_USERTYPE_TRADER = 30000
TAPI_USERTYPE_RISK = 40000
TAPI_USERTYPE_MANAGER = 50000
TAPI_USERTYPE_QUOTE = 60000
TAPI_ACCOUNT_STATE_NORMAL = "N"
TAPI_ACCOUNT_STATE_CANCEL = "C"
TAPI_ACCOUNT_STATE_SLEEP = "S"
TAPI_ACCOUNT_STATE_FROZEN = "F"
TAPI_ACCOUNT_TRADING_RIGHT_NORMAL = "0"
TAPI_ACCOUNT_TRADING_RIGHT_NOTRADE = "1"
TAPI_ACCOUNT_TRADING_RIGHT_CLOSE = "2"
TAPI_ORDER_TYPE_MARKET = "1"
TAPI_ORDER_TYPE_LIMIT = "2"
TAPI_ORDER_TYPE_STOP_MARKET = "3"
TAPI_ORDER_TYPE_STOP_LIMIT = "4"
TAPI_ORDER_TYPE_OPT_EXEC = "5"
TAPI_ORDER_TYPE_OPT_ABANDON = "6"
TAPI_ORDER_TYPE_REQQUOT = "7"
TAPI_ORDER_TYPE_RSPQUOT = "8"
TAPI_ORDER_TYPE_ICEBERG = "9"
TAPI_ORDER_TYPE_GHOST = "A"
TAPI_ORDER_TYPE_HKEX_AUCTION = "B"
TAPI_ORDER_TYPE_SWAP = "C"
TAPI_ORDER_TYPE_LOCK = "D"
TAPI_ORDER_TYPE_UNLOCK = "E"
TAPI_ORDER_TYPE_ENHANCE = "F"
TAPI_ORDER_TYPE_SPECIAL = "G"
TAPI_ORDER_TYPE_LIMITAUCTION = "H"
TAPI_ORDER_SOURCE_SELF_ETRADER = "1"
TAPI_ORDER_SOURCE_PROXY_ETRADER = "2"
TAPI_ORDER_SOURCE_JTRADER = "3"
TAPI_ORDER_SOURCE_MANUAL = "4"
TAPI_ORDER_SOURCE_CARRY = "5"
TAPI_ORDER_SOURCE_PROGRAM = "6"
TAPI_ORDER_SOURCE_DELIVERY = "7"
TAPI_ORDER_SOURCE_ABANDON = "8"
TAPI_ORDER_SOURCE_CHANNEL = "9"
TAPI_ORDER_SOURCE_TEMPORARY = "J"
TAPI_ORDER_SOURCE_PROGRAMMANUAL = "K"
TAPI_ORDER_TIMEINFORCE_GFD = "0"
TAPI_ORDER_TIMEINFORCE_GTC = "1"
TAPI_ORDER_TIMEINFORCE_GTD = "2"
TAPI_ORDER_TIMEINFORCE_FAK = "3"
TAPI_ORDER_TIMEINFORCE_FOK = "4"
TAPI_SIDE_NONE = "N"
TAPI_SIDE_BUY = "B"
TAPI_SIDE_SELL = "S"
TAPI_PositionEffect_NONE = "N"
TAPI_PositionEffect_OPEN = "O"
TAPI_PositionEffect_COVER = "C"
TAPI_PositionEffect_COVER_TODAY = "T"
TAPI_HEDGEFLAG_NONE = "N"
TAPI_HEDGEFLAG_T = "T"
TAPI_HEDGEFLAG_B = "B"
TAPI_HEDGEFLAG_R = "R"
TAPI_ORDER_STATE_SUBMIT = "0"
TAPI_ORDER_STATE_ACCEPT = "1"
TAPI_ORDER_STATE_TRIGGERING = "2"
TAPI_ORDER_STATE_EXCTRIGGERING = "3"
TAPI_ORDER_STATE_QUEUED = "4"
TAPI_ORDER_STATE_PARTFINISHED = "5"
TAPI_ORDER_STATE_FINISHED = "6"
TAPI_ORDER_STATE_CANCELING = "7"
TAPI_ORDER_STATE_MODIFYING = "8"
TAPI_ORDER_STATE_CANCELED = "9"
TAPI_ORDER_STATE_LEFTDELETED = "A"
TAPI_ORDER_STATE_FAIL = "B"
TAPI_ORDER_STATE_DELETED = "C"
TAPI_ORDER_STATE_SUPPENDED = "D"
TAPI_ORDER_STATE_DELETEDFOREXPIRE = "E"
TAPI_ORDER_STATE_EFFECT = "F"
TAPI_ORDER_STATE_APPLY = "G"
TAPI_CALULATE_MODE_COMBINE = "0"
TAPI_CALCULATE_MODE_PERCENTAGE = "1"
TAPI_CALCULATE_MODE_QUOTA = "2"
TAPI_CALCULATE_MODE_CHAPERCENTAGE = "3"
TAPI_CALCULATE_MODE_CHAQUOTA = "4"
TAPI_CALCULATE_MODE_DISCOUNT = "5"
TAPI_MATCH_SOURCE_ALL = "0"
TAPI_MATCH_SOURCE_SELF_ETRADER = "1"
TAPI_MATCH_SOURCE_PROXY_ETRADER = "2"
TAPI_MATCH_SOURCE_JTRADER = "3"
TAPI_MATCH_SOURCE_MANUAL = "4"
TAPI_MATCH_SOURCE_CARRY = "5"
TAPI_MATCH_SOURCE_PROGRAM = "6"
TAPI_MATCH_SOURCE_DELIVERY = "7"
TAPI_MATCH_SOURCE_ABANDON = "8"
TAPI_MATCH_SOURCE_CHANNEL = "9"
TAPI_MATCH_SOURCE_BLOOMBERG = "B"
TAPI_MATCH_SOURCE_AUTOPHONE = "A"
TAPI_MATCH_SOURCE_GIVEUP = "C"
TAPI_MATCH_SOURCE_EXERCISE = "E"
TAPI_MATCH_SOURCE_TAKEUP = "D"
TAPI_MATCH_SOURCE_INPUT_DELETE = "#"
TAPI_MATCH_SOURCE_ERRORADJUST = "F"
TAPI_MATCH_SOURCE_BROKER_ETRADER = "G"
TAPI_MATCH_SOURCE_TEMPORARY = "J"
TAPI_MATCH_SOURCE_PROGRAMMANUAL = "K"
TAPI_CLOSE_MODE_NONE = "N"
TAPI_CLOSE_MODE_UNFINISHED = "U"
TAPI_CLOSE_MODE_OPENCOVER = "C"
TAPI_CLOSE_MODE_CLOSETODAY = "T"
TAPI_FUTURES_ALG_ZHUBI = "Y"
TAPI_FUTURES_ALG_DINGSHI = "N"
TAPI_OPTION_ALG_FUTURES = "1"
TAPI_OPTION_ALG_OPTION = "2"
TAPI_LWFlag_L = "L"
TAPI_LWFlag_W = "W"
TAPI_CASHINOUT_MODE_FEEADJUST = "0"
TAPI_CASHINOUT_MODE_YKADJUST = "1"
TAPI_CASHINOUT_MODE_PLEDGE = "2"
TAPI_CASHINOUT_MODE_INTERESTREVENUE = "3"
TAPI_CASHINOUT_MODE_COLLECTIONCOST = "4"
TAPI_CASHINOUT_MODE_OTHER = "5"
TAPI_CASHINOUT_MODE_COMPANY = "6"
TAPI_DEPOSITCALCULATE_MODE_FEN = "1"
TAPI_DEPOSITCALCULATE_MODE_SUO = "2"
TAPI_CMB_DIRECT_FIRST = "1"
TAPI_CMB_DIRECT_SECOND = "2"
TAPI_DELIVERY_MODE_GOODS = "G"
TAPI_DELIVERY_MODE_CASH = "C"
TAPI_DELIVERY_MODE_EXECUTE = "E"
TAPI_DELIVERY_MODE_ABANDON = "A"
TAPI_DELIVERY_MODE_HKF = "H"
TAPI_CONTRACT_TYPE_TRADEQUOTE = "1"
TAPI_CONTRACT_TYPE_QUOTE = "2"
TAPI_TACTICS_TYPE_NONE = "N"
TAPI_TACTICS_TYPE_READY = "M"
TAPI_TACTICS_TYPE_ATUO = "A"
TAPI_TACTICS_TYPE_CONDITION = "C"
APIORDER_INSERT = "1"
APIORDER_MODIFY = "2"
APIORDER_DELETE = "3"
APIORDER_SUSPEND = "4"
APIORDER_ACTIVATE = "5"
APIORDER_SYSTEM_DELETE = "6"
TAPI_TRIGGER_CONDITION_NONE = "N"
TAPI_TRIGGER_CONDITION_GREAT = "G"
TAPI_TRIGGER_CONDITION_LITTLE = "L"
TAPI_TRIGGER_PRICE_NONE = "N"
TAPI_TRIGGER_PRICE_BUY = "B"
TAPI_TRIGGER_PRICE_SELL = "S"
TAPI_TRIGGER_PRICE_LAST = "L"
TAPI_TRADE_STATE_BID = "1"
TAPI_TRADE_STATE_MATCH = "2"
TAPI_TRADE_STATE_CONTINUOUS = "3"
TAPI_TRADE_STATE_PAUSED = "4"
TAPI_TRADE_STATE_CLOSE = "5"
TAPI_TRADE_STATE_DEALLAST = "6"
TAPI_TRADE_STATE_GWDISCONNECT = "0"
TAPI_TRADE_STATE_UNKNOWN = "N"
TAPI_TRADE_STATE_INITIALIZE = "I"
TAPI_TRADE_STATE_READY = "R"
TAPI_NOTICE_IGNORE_NONE = 0x00000000
TAPI_NOTICE_IGNORE_ALL = 0xFFFFFFFF
TAPI_NOTICE_IGNORE_FUND = 0x00000001
TAPI_NOTICE_IGNORE_ORDER = 0x00000002
TAPI_NOTICE_IGNORE_FILL = 0x00000004
TAPI_NOTICE_IGNORE_POSITION = 0x00000008
TAPI_NOTICE_IGNORE_CLOSE = 0x00000010
TAPI_NOTICE_IGNORE_POSITIONPROFIT = 0x00000020
TAPI_ORDER_QRY_TYPE_ALL = "A"
TAPI_ORDER_QRY_TYPE_UNENDED = "U"
TAPI_LOGINTYPE_NORMAL = "N"
TAPI_LOGINTYPE_TEMPORARY = "T"
TAPI_MSG_RECEIVER_ACCOUNTNO = "1"
TAPI_MSG_RECEIVER_ACCOUNTGROUPNO = "2"
TAPI_MSG_RECEIVER_ATTRIBUTE = "3"
TAPI_MSG_RECEIVER_USERNO = "4"
TAPI_MSG_LEVEL_NORMAL = "1"
TAPI_MSG_LEVEL_IMPORTANT = "2"
TAPI_MSG_LEVEL_IMERGENCY = "3"
TAPI_Msg_TYPE_MANAGER = "1"
TAPI_Msg_TYPE_RISKCONTROL = "2"
TAPI_BILL_DATE = "D"
TAPI_BILL_MONTH = "M"
TAPI_BILL_FILE_TXT = "T"
TAPI_BILL_FILE_PDF = "F"
SettleFlag_AutoSettle = "0"
SettleFlagh_Manual = "2"
TAPI_ABNORMAL_NORMAL = "0"
TAPI_ABNORMAL_GATHERINFO_NONE = "1"
TAPI_ABNORMAL_AUTHKEYVERSION = "2"
TAPI_ABNORMAL_GATHERINFO_WRONG = "3"
TAPI_STRATEGY_C_BULL = "CNSJC"
TAPI_STRATEGY_P_BEAR = "PXSJC"
TAPI_STRATEGY_P_BULL = "PNSJC"
TAPI_STRATEGY_C_BEAR = "CXSJC"
TAPI_STRATEGY_S_STRADDLE = "KS"
TAPI_STRATEGY_S_STRANGLE = "KKS"
TAPI_STRATEGY_ZBD = "ZBD"
TAPI_STRATEGY_ZXJ = "ZXJ"
TAPI_STRATEGY_COMBINE = "1"
TAPI_STRATEGY_SPLIT = "2"
TAPI_SPOT_LOCK = "3"
TAPI_SPOT_UNLOCK = "4"
TAPI_OPTION_EXERCISE = "5"
TAPI_OPTION_EXERCISE_COMBINE = "6"
TAPI_APPLYTYPE_CASH = "1"
TAPI_APPLYTYPE_LOAN = "2"
TAPI_APPLYSTAUTS_APPLY = "0"
TAPI_APPLYSTAUTS_FAIL = "1"
TAPI_APPLYSTAUTS_WAIT = "2"
TAPI_APPLYSTAUTS_CANCEL = "3"
TAPI_APPLYSTAUTS_DONE = "4"
TAPI_APPLYSTAUTS_NOLUCKY = "5"
TAPI_APPLYSTAUTS_LUCKY = "6"
TAPI_APPLYSTAUTS_WAITCANCEL = "7"
TAPI_CERTIFICATE_MAINLANDID = "1"
TAPI_CERTIFICATE_MILITARYCARD = "2"
TAPI_CERTIFICATE_POLICECARD = "3"
TAPI_CERTIFICATE_SOLDIERID = "4"
TAPI_CERTIFICATE_ACCOUNTBOOK = "5"
TAPI_CERTIFICATE_PASSPORT_CHINESE = "6"
TAPI_CERTIFICATE_MTPS = "7"
TAPI_CERTIFICATE_HOMERETURN = "8"
TAPI_CERTIFICATE_BUSINESSLICENSE = "9"
TAPI_CERTIFICATE_ENTERPRISELEGALPESON = "A"
TAPI_CERTIFICATE_OTHERID = "B"
TAPI_CERTIFICATE_PASSPORT_OTHER = "C"
TAPI_CERTIFICATE_OTHER = "x"
TAPI_CERTIFICATE_BUSINESSREGISTRATION = "R"

TAPIERROR_LOGIN = 10001
TAPIERROR_LOGIN_USER = 10002
TAPIERROR_LOGIN_DDA = 10003
TAPIERROR_LOGIN_LICENSE = 10004
TAPIERROR_LOGIN_MODULE = 10005
TAPIERROR_LOGIN_FORCE = 10006
TAPIERROR_LOGIN_STATE = 10007
TAPIERROR_LOGIN_PASS = 10008
TAPIERROR_LOGIN_RIGHT = 10009
TAPIERROR_LOGIN_COUNT = 10010
TAPIERROR_LOGIN_NOTIN_SERVERFLAGUSRES = 10011
TAPIERROR_LOGIN_FREEZE = 10012
TAPIERROR_LOGIN_TOFREEZE = 10013
TAPIERROR_LOGIN_ACCOUNTSTATE = 10014
TAPIERROR_LOGIN_SECCERTIFI = 10015
TAPIERROR_LOGIN_NOSECONDSET = 10016
TAPIERROR_LOGIN_NOTURSTHOST = 10017
TAPIERROR_LOGIN_NOTINTRADECENTER = 10019
TAPIERROR_LOGIN_INCONSISTENT = 10020
TAPIERROR_LOGIN_NOCENTERFRONTADDRESS = 10021
TAPIERROR_LOGIN_PROHIBITACCOUNTTYPE = 10022
TAPIERROR_LOGIN_GATHERINFO_DIRECT = 10023
TAPIERROR_LOGIN_GATHERINFO_RELAY = 10024
TAPIERROR_LOGIN_RESET_PASSWORD = 10025
TAPIERROR_LOGIN_RESET_PASSWORD_EXCEEDED = 10026
TAPIERROR_LOGIN_OPERATOR_UNALLOWED_RESET = 10027
TAPIERROR_LOGIN_RESET_PASSWORD_FROZEN = 10028
TAPIERROR_LOGIN_VERIFYIDENTITY_FAILED = 10029
TAPIERROR_LOGIN_VERIFYIDENTITY_EXCEED = 10030
TAPIERROR_LOGIN_LICENSE_EXPIRED = 10031
TAPIERROR_LOGIN_PROHIBITRESETPASSWORD = 10032
TAPIERROR_LOGIN_UNFREEZE = 10033
TAPIERROR_LOGIN_UNFREEZE_EXCEEDED = 10034
TAPIERROR_LOGIN_PROHIBIT_UNFREEZE = 10035
TAPIERROR_LOGIN_NONEED_UNFREEZE = 10036
TAPIERROR_SECONDCERTIFICATION_FAIL = 14001
TAPIERROR_SECONDCERTIFICATION_TIMEOVER = 14002
TAPIERROR_SECONDCERTIFICATION_RELOGIN = 14003
TAPIERROR_SECONDCERTIFICATION_FREEZE = 14004
TAPIERROR_DATA_REPEAT = 1062
TAPIERROR_CONN_DATABASE = 11000
TAPIERROR_OPER_DATABASE = 11001
TAPIERROR_NEED_ONETOONE = 11002
TAPIERROR_EXIST_RELATEINFO = 11003
TAPIERROR_EXIST_RELATEINFOOFGROUP = 11004
TAPIERROR_CHECK_FAILED = 11006
TAPIERROR_EXIST_OUTSIDECOMMODITYNO = 11007
TAPIERROR_NOTEXIST_BILL = 11008
TAPIERROR_LOGIN_PROHIBITADDACCOUNTTYPE = 11009
TAPIERROR_ACCOUNTINFO_NOTEXPTY = 11010
TAPIERROR_ACCOUNTINFO_SuperiorNOTEMPTY = 11011
TAPIERROR_USERTRUSTDEVICE_ADDLIMITE = 11012
TAPIERROR_UPPERACCOUNTMAPPING_NOTALLOW = 11013
TAPIERROR_USERPASSWORD_MOD_SOURCE = 12001
TAPIERROR_USERPASSWORD_MOD_SAME = 12002
TAPIERROR_USERPASSWORD_MOD_COMPLEXITY = 12003
TAPIERROR_CURRENCY_ONLY_ONEBASE = 13001
TAPIERROR_CURRENCY_ONLY_USDHKD = 13002
TAPIERROR_GATHERINFO_NO_AUTHKEY = 17001
TAPIERROR_GATHERINFO_AUTH_FAILED = 17002
TAPIERROR_ORDERINSERT_ACCOUNT = 60001
TAPIERROR_ORDERINSERT_ACCOUNT_STATE = 60002
TAPIERROR_ORDERINSERT_TRADECENT_ERROR = 60003
TAPIERROR_ORDERINT_MAINACCOUNT_ERROR = 60004
TAPIERROR_ORDERINT_MAINACCINFO_ERROR = 60005
TAPIERROR_ORDERINT_NO_OPTMARKET_ERROR = 60006
TAPIERROR_ORDERINT_UN_SUPPORT_ERROR = 60007
TAPIERROR_ORDERINSERT_CONTRACT = 60011
TAPIERROR_ORDERINSERT_LME_NOTREADY = 60012
TAPIERROR_ORDERINSERT_ERROR_ORDER_TYPE = 60013
TAPIERROR_ORDERINSERT_READY_TYPE_ERROR = 60014
TAPIERROR_ORDERINSERT_ORDER_TYPE_ERROR = 60015
TAPIERROR_ORDERINSERT_SUBCONTRACT = 60016
TAPIERROR_ORDERINSERT_SPOTBUYQTY = 60017
TAPIERROR_ORDERINSERT_RESERVE_PUT = 60018
TAPIERROR_ORDERINSERT_RESERVE_B_OPEN = 60019
TAPIERROR_ORDERINSERT_RESERVE_S_COVER = 60020
TAPIERROR_ORDER_NOTRADE_ACCOUNT = 60021
TAPIERROR_ORDER_NOTRADE_COM_GROUP = 60022
TAPIERROR_ORDER_NOTRADE_ACC_CONTRACT = 60023
TAPIERROR_ORDER_NOTRADE_SYSTEM = 60024
TAPIERROR_ORDER_CLOSE_ACCOUNT = 60025
TAPIERROR_ORDER_CLOSE_ACC_CONTRACT = 60026
TAPIERROR_ORDER_CLOSE_SYSTEM = 60027
TAPIERROR_ORDER_CLOSE_DAYS = 60028
TAPIERROR_ORDER_NOTRADE_RISK = 60029
TAPIERROR_ORDER_CLOSE_RISK = 60030
TAPIERROR_ORDERINSERT_POSITIONMAX = 60031
TAPIERROR_ORDERINSERT_ONCEMAX = 60032
TAPIERROR_ORDERINSERT_TRADEROUTE = 60033
TAPIERROR_ORDER_IN_MOD_PRICE_ERROR = 60034
TAPIERROR_ORDER_IN_GIVEUP_POS_MAX = 60035
TAIERROR_ORDER_ETF_POSITIONMAX = 60036
TAIERROR_ORDER_ETF_BUYPOSITIONMAX = 60037
TAIERROR_ORDER_ETF_BUYONEDAYPOSITIONMAX = 60038
TAIERROR_ORDERINSERT_NOTENOUGHSPOT = 60039
TAIERROR_ORDERINSERT_RESERVE_SPOT = 60040
TAPIERROR_UPPERCHANNEL_NOT_LOGIN = 60041
TAPIERROR_UPPERCHANNEL_NOT_FOUND = 60042
TAPIERROR_COMMODITY_LOCK = 60043
TAPIERROR_SPOT_ROOT_COMMODITY = 60044
TAIERROR_SPOT_ROOTCOM_TRADEROUTE = 60045
TAPIERROR_ORDERINSERT_NOTENOUGHFUND = 60051
TAPIERROR_ORDERINSERT_FEE = 60052
TAPIERROR_ORDERINSERT_MARGIN = 60053
TAPIERROR_ORDERINSERT_BASENOFUND = 60054
TAPIERROR_ORDERINSERT_MARGINAMOUNT = 60055
TAPIERROR_ORDERINSERT_OPENRATIO = 60056
TAPIERROR_ORDERINSERT_GROUP_OPENRATIO = 60057
TAPIERROR_ORDERINSERT_RISKARRAY = 60058
TAIERROR_ORDERINSERT_BUYLIMITE = 60059
TAIERROR_ORDERINSERT_GROUP_BUYLIMITE = 60060
TAPIERROR_ORDERDELETE_NOT_SYSNO = 60061
TAPIERROR_ORDERDELETE_NOT_STATE = 60062
TAPIERROR_ORDERDELETE_NO_INPUT = 60063
TAPIERROR_ORDERDELETE_NO_TRADE = 60064
TAPIERROR_ORDERMODIFY_NOT_STATE = 60071
TAPIERROR_ORDERMODIFY_BACK_INPUT = 60072
TAPIERROR_ORDERMODIFY_RISK_ORDER = 60073
TAPIERROR_ORDERMODIFY_ERROR_QTY = 60074
TAPIERROR_ORDERMODIFY_ERROR_READY = 60075
TAPIERROR_ORDERINPUT_CANNOTMOVE = 60081
TAPIERROR_ORDERINPUT_REPEAT = 60091
TAPIERROR_CONTRACT_QUOTE = 60101
TAPIERROR_UPPER_ONCEMAX = 60111
TAPIERROR_UPPER_POSITIONMAX = 60112
TAPIERROR_ORDERINSERT_POSMAX_COM = 60113
TAPIERROR_ORDERINSERT_POSMAX_COM_NET = 60114
TAPIERROR_ORDERINSERT_CLOSEMODE = 60121
TAPIERROR_CLOSE_ORDER = 60122
TAPIERROR_CLOSE_MATCH = 60123
TAPIERROR_CLOSE_SPOT_OUT_LOCK = 60124
TAPIERROR_CLOSE_SPOT_OUT_NULL = 60125
TAPIERROR_MOD_DEL_NO_ORDER = 60131
TAPIERROR_MOD_DEL_GATEWAY_DISCON = 60132
TAPIERROR_MATCHINPUT_REPEAT = 60141
TAPIERROR_MATCHINPUT_NO_ORDER = 60142
TAPIERROR_MATCHINPUT_NO_CONTRACT = 60143
TAPIERROR_MATCHINPUT_PARM_ERROR = 60144
TAPIERROR_MATCHINPUT_OSTATE_ERROR = 60145
TAPIERROR_MATCHINPUT_OCMODE_ERROR = 60146
TAPIERROR_MATCHREMOVE_NO_MATCH = 60151
TAPIERROR_MATCHREMOVE_STATE_ERROR = 60152
TAPIERROR_MATCHINFO_CONTRACT_ERROR = 60153
TAPIERROR_ORDERINPUT_STATE_ERROR = 60161
TAPIERROR_ORDERINPUT_MOD_ERROR = 60162
TAPIERROR_ORDERREMOVE_ERROR = 60163
TAPIERROR_ORDERINPUT_MOD_STATE_ERROR = 60164
TAPIERROR_ORDEREXCHANGE_STATE_ERROR = 60165
TAPIERROR_ORDERREMOVE_NOT_ERROR = 60166
TAPIERROR_ORDERMARKET_DELETE_NOTFOUND = 60171
TAPIERROR_ORDERMARKET_DEL_ACCOUNT_NE = 60172
TAPIERROR_ORDERMARKET_DEL_COMMODITY_NE = 60173
TAPIERROR_ORDERMARKET_DEL_CONTRACT_NE = 60174
TAPIERROR_ORDERMARKET_DEL_SIDE_EQ = 60175
TAPIERROR_ORDERMARKET_DEL_SIDE_ERROR = 60176
TAPIERROR_ORDERMARKET_OTHER_SIDE_ERROR = 60177
TAPIERROR_ORDERMARKET_DEL_SYSTEMNO_ERROR = 60178
TAPIERROR_ORDERACTIVATE_NOTFOUND_ERROR = 60181
TAPIERROR_ORDERACTIVATE_STATE_ERROR = 60182
TAPIERROR_ORDERACTIVATE_GATEWAY_ERROR = 60183
TAPIERROR_TRANSIT_ORDERINSERT_RIGHT = 60191
TAPIERROR_TRANSIT_ORDERINSERT_DISCON = 60192
TAPIERROR_TRANSIT_ORDERINSERT_DISCON_DEST = 60193
TAPIERROR_TRANSIT_ORDERDELETE_DISCON_DEST = 60194
TAPIERROR_TRANSIT_ORDERMODIFY_DISCON_DEST = 60195
TAPIERROR_TRANSIT_ORDER_OPERATOR = 60196
TAPIERROR_ORDER_DISALLOWBUY_ACCOUNT = 60201
TAPIERROR_ORDER_DISALLOWSELL_ACCOUNT = 60202
TAPIERROR_ORDER_DISALLOWBUY_SYSTEM = 60203
TAPIERROR_ORDER_DISALLOWSELL_SYSTEM = 60204
TAPIERROR_ORDER_DIS_SELLOPTION_ACCOUNT = 60205
TAPIERROR_ORDER_DIS_SELLOPTION_SYSTEM = 60206
TAPIERROR_ORDER_CONTRACT_CLOSE = 60207
TAPIERROR_ORDER_INSERT_RISK_CLOSE = 60208
TAPIERROR_ORDERINSERT_LOANAMOUNT = 60211
TAPIERROR_COMBINE_COMMODITY = 60220
TAPIERROR_COMBINE_CALLORPUT = 60221
TAPIERROR_COMBINE_COMPOSITION = 60222
TAPIERROR_COMBINE_COMPOSITION_QTY = 60223
TAPIERROR_COMBINE_HEDGEFLAG = 60224
TAPIERROR_COMBINE_ORDERSIDE = 60225
TAPIERROR_COMBINE_CONTRACTSIZE = 60226
TAPIERROR_COMBINE_CONTRACTDAYS = 60227
TAPIERROR_COMBINE_STRIKEPRICE = 60228
TAPIERROR_COMBINE_CONTRACT_SAME = 60229
TAPIERROR_ORDERINSERT_UNLOCK_NOE = 60230
TAPIERROR_ORDEROPEN_OPT_SPOT_NOE = 60231
TAPIERROR_ORDERCLOSE_OPT_SPOT_NOE = 60232
TAPIERROR_ORDERINSERT_COVERED_UNVLD = 60233
TAPIERROR_ORDERINSERT_QUOTE_PRICE = 60240
TAPIERROR_ACCOUNTIPO_REPEAT = 60250
TAPIERROR_ACCOUNTIPO_NOT_EXIST = 60251
TAPIERROR_ORDERINSERT_MAIN_MARGIN = 60261
TAPIERROR_GW_NOT_READY = 80001
TAPIERROR_GW_INVALID_COMMODITY = 80002
TAPIERROR_GW_INVALID_CONTRACT = 80003
TAPIERROR_GW_INVALID_FIELD = 80004
TAPIERROR_GW_INVALID_PRICE = 80005
TAPIERROR_GW_INVALID_VOLUME = 80006
TAPIERROR_GW_INVALID_TYPE = 80007
TAPIERROR_GW_INVALID_MODE = 80008
TAPIERROR_GW_ORDER_NOT_EXIST = 80009
TAPIERROR_GW_SEND_FAIL = 80010
TAPIERROR_GW_REJ_BYUPPER = 80011
TAPIERROR_TRADEFRONT_MODULETYPEERR = 90001
TAPIERROR_TRADEFRONT_TOOMANYDATA = 90002
TAPIERROR_TRADEFRONT_NODATA = 90003
TAPIERROT_TRADEFRONT_NOUSER = 90004
TAPIERROR_TRADEFRONT_DISCONNECT_TRADE = 90011
TAPIERROR_TRADEFRONT_DISCONNECT_MANAGE = 90012
TAPIERROR_TRADEFRONT_ACCOUNT = 90021
TAPIERROR_TRADEFRONT_ORDER = 90022
TAPIERROR_TRADEFRONT_FREQUENCY = 90023
TAPIERROR_TRADEFRONT_RUFUSE = 90024
TAPIERROR_TRADEFRONT_SELFMATCH = 90025
TAPIERROR_TRADEFRONT_DELETEFORCE = 90026
TAPIERROR_SUCCEED = 0
TAPIERROR_ConnectFail = -1
TAPIERROR_LinkAuthFail = -2
TAPIERROR_HostUnavailable = -3
TAPIERROR_SendDataError = -4
TAPIERROR_TestIDError = -5
TAPIERROR_NotReadyTestNetwork = -6
TAPIERROR_CurTestNotOver = -7
TAPIERROR_NOFrontAvailable = -8
TAPIERROR_DataPathAvaiable = -9
TAPIERROR_RepeatLogin = -10
TAPIERROR_InnerError = -11
TAPIERROR_LastReqNotFinish = -12
TAPIERROR_InputValueError = -13
TAPIERROR_AuthCode_Invalid = -14
TAPIERROR_AuthCode_Expired = -15
TAPIERROR_AuthCode_TypeNotMatch = -16
TAPIERROR_API_NotReady = -17
TAPIERROR_UDP_LISTEN_FAILED = -18
TAPIERROR_UDP_LISTENING = -19
TAPIERROR_NotImplemented = -20
TAPIERROR_CallOneTimeOnly = -21
TAPIERROR_ORDER_FREQUENCY = -22
TAPIERROR_RENTQRY_TOOFAST = -23
TAPIERROR_CALL_NOCONDITION = -24
TAPIERROR_ORDER_NOTFOUND = -25
TAPIERROR_LOGPATH_EMPTY = -26
TAPIERROR_LOGPATH_FAILOPEN = -27
TAPIERROR_RIGHT_TRADER = -28
TAPIERROR_RIGHT_ORDERINPUT = -29
TAPIERROR_RIGHT_LOCALOPERATION = -30
TAPIERROR_RIGHT_ORDERTRANSFER = -31
TAPIERROR_FILLINPUT_SYSTEMNO = -32
TAPIERROR_FILLREMOVE_MATCHNO = -33
TAPIERROR_FILLREQMOVE_NOFUND = -34
TAPIERROR_LOCALMODIFY_ACCOUNT = -35
TAPIERROR_LOCALTRANSFER_ACCOUNT = -36
TAPIERROR_INPUTERROR_PHONE = -37
TAPIERROR_ERROR_CONTACT = -38
TAPIERROR_ERROR_REJESTVERTIFICATE = -39
TAPIERROR_ERROR_NOTREQUESTSECONDCODE = -44
TAPIERROR_RIGHT_SETPASSWORD = -40
TAPIERROR_RISK_OPERERROR = -41
TAPIERROR_ORDER_MODACCOUNT = -42
TAPIERROR_MEMORY_ALLOCFAILED = -43
TAPIERROR_ERROR_LICENSECOMMODITY = -45
TAPIERROR_ERROR_SYSTEMTYPE = -56
TAPIERROR_ERROR_RECONNECTING = -57
TAPIERROR_UNFREEZE_SUCCESS = -58
TAPIERROR_HISDATA_DAYS = -59
TAPIERROR_GATHERINFO_NORELAY = -46
TAPIERROR_GATHERINFO_PARTY = -47
TAPIERROR_GATHERINFO_DATALOAD = -48
TAPIERROR_ERROR_PRICETICK = -50
TAPIERROR_OVER_MAXLOANRATIO = -52
TAPIERROR_OVER_MAXLOANVALUE = -53
TAPIERROR_OVER_APPLYQTY = -54
TAPIERROR_NULL_IPOINFO = -55
TAPIERROR_INPUTERROR_NULL = -10000
TAPIERROR_INPUTERROR_TAPIYNFLAG = -10001
TAPIERROR_INPUTERROR_TAPILOGLEVEL = -10002
TAPIERROR_INPUTERROR_TAPICommodityType = -10003
TAPIERROR_INPUTERROR_TAPICallOrPutFlagType = -10004
TAPIERROR_INPUTERROR_TAPIBucketDateFlag = -11001
TAPIERROR_INPUTERROR_TAPIHisQuoteType = -11002
TAPIERROR_INPUTERROR_TAPIAccountType = -12001
TAPIERROR_INPUTERROR_TAPIUserTypeType = -12002
TAPIERROR_INPUTERROR_TAPIAccountState = -12003
TAPIERROR_INPUTERROR_TAPIAccountFamilyType = -12004
TAPIERROR_INPUTERROR_TAPIOrderTypeType = -12005
TAPIERROR_INPUTERROR_TAPIOrderSourceType = -12006
TAPIERROR_INPUTERROR_TAPITimeInForceType = -12007
TAPIERROR_INPUTERROR_TAPISideType = -12008
TAPIERROR_INPUTERROR_TAPIPositionEffectType = -12009
TAPIERROR_INPUTERROR_TAPIHedgeFlagType = -12010
TAPIERROR_INPUTERROR_TAPIOrderStateType = -12011
TAPIERROR_INPUTERROR_TAPICalculateModeType = -12012
TAPIERROR_INPUTERROR_TAPIMatchSourceType = -12013
TAPIERROR_INPUTERROR_TAPIOpenCloseModeType = -12014
TAPIERROR_INPUTERROR_TAPIFutureAlgType = -12015
TAPIERROR_INPUTERROR_TAPIOptionAlgType = -12016
TAPIERROR_INPUTERROR_TAPIBankAccountLWFlagType = -12017
TAPIERROR_INPUTERROR_TAPIBankAccountStateType = -12018
TAPIERROR_INPUTERROR_TAPIBankAccountSwapStateType = -12019
TAPIERROR_INPUTERROR_TAPIBankAccountTransferStateType = -12020
TAPIERROR_INPUTERROR_TAPIMarginCalculateModeType = -12021
TAPIERROR_INPUTERROR_TAPIOptionMarginCalculateModeType = -12022
TAPIERROR_INPUTERROR_TAPICmbDirectType = -12023
TAPIERROR_INPUTERROR_TAPIDeliveryModeType = -12024
TAPIERROR_INPUTERROR_TAPIContractTypeType = -12025
TAPIERROR_INPUTERROR_TAPIPartyTypeType = -12026
TAPIERROR_INPUTERROR_TAPIPartyCertificateTypeType = -12027
TAPIERROR_INPUTERROR_TAPIMsgReceiverType = -12028
TAPIERROR_INPUTERROR_TAPIMsgTypeType = -12029
TAPIERROR_INPUTERROR_TAPIMsgLevelType = -12030
TAPIERROR_INPUTERROR_TAPITransferDirectType = -12031
TAPIERROR_INPUTERROR_TAPITransferStateType = -12032
TAPIERROR_INPUTERROR_TAPITransferTypeType = -12033
TAPIERROR_INPUTERROR_TAPITransferDeviceIDType = -12034
TAPIERROR_INPUTERROR_TAPITacticsTypeType = -12035
TAPIERROR_INPUTERROR_TAPIORDERACT = -12036
TAPIERROR_INPUTERROR_TAPIBillTypeType = -12037
TAPIERROR_INPUTERROR_TAPIBillFileTypeType = -12038
TAPIERROR_INPUTERROR_TAPIOFFFlagType = -12039
TAPIERROR_INPUTERROR_TAPICashAdjustTypeType = -12040
TAPIERROR_INPUTERROR_TAPITriggerConditionType = -12041
TAPIERROR_INPUTERROR_TAPITriggerPriceTypeType = -12042
TAPIERROR_INPUTERROR_TAPITradingStateType = -12043
TAPIERROR_INPUTERROR_TAPIMarketLevelType = -12044
TAPIERROR_INPUTERROR_TAPIOrderQryTypeType = -12045
TAPIERROR_INPUTERROR_TAPIClientID = -12046
TAPIERROR_INPUTERROR_QryHisQuoteParam = -13001
TAPIERROR_INPUTERROR_TAPIIncludeNAN = -13002
TAPIERROR_INPUTERROR_TAPIPasswordType = -12048
TAPIERROR_INPUTERROR_TAPISettleFlagType = -12049
TAPIERROR_INPUTERROR_TAPILoginTypeType = -12050
TAPIERROR_INPUTERROR_TapAPISpecialOrderTypeType = -12051
TAPIERROR_INPUTERROR_TapAPICombineStrategyType = -12052
TAPIERROR_INPUTERROR_TAPIClientIDType = -12053
TAPIERROR_INPUTERROR_TAPIClientLocationIDType = -12054
TAPIERROR_INPUTERROR_TapAPIApplyTypeType = -12055
TAPIERROR_INPUTERROR_TAPIDATE = -12056
TAPIERROR_DISCONNECT_CLOSE_INIT = 1
TAPIERROR_DISCONNECT_CLOSE_PASS = 2
TAPIERROR_DISCONNECT_READ_ERROR = 3
TAPIERROR_DISCONNECT_WRITE_ERROR = 4
TAPIERROR_DISCONNECT_BUF_FULL = 5
TAPIERROR_DISCONNECT_IOCP_ERROR = 6
TAPIERROR_DISCONNECT_PARSE_ERROR = 7
TAPIERROR_DISCONNECT_CONNECT_TIMEOUT = 8
TAPIERROR_DISCONNECT_INIT_ERROR = 9
TAPIERROR_DISCONNECT_HAS_CONNECTED = 10
TAPIERROR_DISCONNECT_HAS_EXIT = 11
TAPIERROR_DISCONNECT_TRY_LATER = 12



from dataclasses import dataclass
from datetime import datetime
from pathlib import Path
from typing import Any, Dict, Tuple, List

from vnpy.event import EventEngine
from vnpy.trader.utility import get_folder_path, ZoneInfo
from vnpy.trader.constant import (
    Exchange,
    Product,
    Direction,
    Status,
    OrderType,
    OptionType
)
from vnpy.trader.gateway import BaseGateway
from vnpy.trader.object import (
    CancelRequest,
    OrderRequest,
    SubscribeRequest,
    TickData,
    ContractData,
    OrderData,
    TradeData,
    PositionData,
    AccountData
)

from ..api import (
    MdApi,
    TdApi,
    APIYNFLAG_NO,
    TAPI_CALLPUT_FLAG_CALL,
    TAPI_CALLPUT_FLAG_PUT,
    TAPI_CALLPUT_FLAG_NONE,
    TAPIERROR_SUCCEED,
    APILOGLEVEL_NONE,
    TAPI_COMMODITY_TYPE_FUTURES,
    TAPI_COMMODITY_TYPE_OPTION,
    TAPI_ORDER_TYPE_MARKET,
    TAPI_ORDER_TYPE_LIMIT,
    TAPI_SIDE_NONE,
    TAPI_SIDE_BUY,
    TAPI_SIDE_SELL,
    TAPI_ORDER_STATE_SUBMIT,
    TAPI_ORDER_STATE_ACCEPT,
    TAPI_ORDER_STATE_QUEUED,
    TAPI_ORDER_STATE_PARTFINISHED,
    TAPI_ORDER_STATE_FINISHED,
    TAPI_ORDER_STATE_CANCELED,
    TAPI_ORDER_STATE_LEFTDELETED,
    TAPI_ORDER_STATE_FAIL
)


# 委托状态映射
STATUS_TAP2VT: Dict[str, Status] = {
    TAPI_ORDER_STATE_SUBMIT: Status.SUBMITTING,
    TAPI_ORDER_STATE_ACCEPT: Status.SUBMITTING,
    TAPI_ORDER_STATE_QUEUED: Status.NOTTRADED,
    TAPI_ORDER_STATE_PARTFINISHED: Status.PARTTRADED,
    TAPI_ORDER_STATE_FINISHED: Status.ALLTRADED,
    TAPI_ORDER_STATE_CANCELED: Status.CANCELLED,
    TAPI_ORDER_STATE_LEFTDELETED: Status.CANCELLED,
    TAPI_ORDER_STATE_FAIL: Status.REJECTED,
}

# 多空方向映射
DIRECTION_TAP2VT: Dict[str, Direction] = {
    TAPI_SIDE_NONE: Direction.NET,
    TAPI_SIDE_BUY: Direction.LONG,
    TAPI_SIDE_SELL: Direction.SHORT,
}
DIRECTION_VT2TAP: Dict[Direction, str] = {v: k for k, v in DIRECTION_TAP2VT.items()}

# 委托类型映射
ORDERTYPE_TAP2VT: Dict[str, OrderType] = {
    TAPI_ORDER_TYPE_MARKET: OrderType.MARKET,
    TAPI_ORDER_TYPE_LIMIT: OrderType.LIMIT
}
ORDERTYPE_VT2TAP = {v: k for k, v in ORDERTYPE_TAP2VT.items()}

# 交易所映射
EXCHANGE_TAP2VT: Dict[str, Exchange] = {
    "SGX": Exchange.SGX,
    "INE": Exchange.INE,
    "APEX": Exchange.APEX,
    "NYMEX": Exchange.NYMEX,
    "LME": Exchange.LME,
    "COMEX": Exchange.COMEX,
    "CBOT": Exchange.CBOT,
    "HKEX": Exchange.HKFE,
    "CME": Exchange.CME,
    "TOCOM": Exchange.TOCOM,
    "KRX": Exchange.KRX,
    "ICUS": Exchange.ICE,
    "ICEU": Exchange.ICE
}
EXCHANGE_VT2TAP: Dict[Exchange, str] = {v: k for k, v in EXCHANGE_TAP2VT.items()}

# 产品类型映射
Product_TAP2VT: Dict[str, Product] = {
    TAPI_COMMODITY_TYPE_FUTURES: Product.FUTURES,
    TAPI_COMMODITY_TYPE_OPTION: Product.OPTION
}

# 期权类型映射
OPTIONTYPE_TAP2VT: Dict[str, OptionType] = {
    TAPI_CALLPUT_FLAG_CALL: OptionType.CALL,
    TAPI_CALLPUT_FLAG_PUT: OptionType.PUT
}
OPTIONTYPE_VT2TAP: Dict[OptionType, str] = {v: k for k, v in OPTIONTYPE_TAP2VT.items()}

# 其他常量
CHINA_TZ = ZoneInfo("Asia/Shanghai")

# 合约数据全局缓存字典
commodity_infos: Dict[tuple[str, str, str], "CommodityInfo"] = {}
contract_infos: Dict[Tuple[str, "Exchange"], "ContractInfo"] = {}
option_contract_map: Dict[str, ContractData] = {}


class TapGateway(BaseGateway):
    """
    VeighNa用于对接易盛9.0外盘的交易接口。
    """

    default_name: str = "TAP"

    default_setting: Dict[str, Any] = {
        "行情账号": "",
        "行情密码": "",
        "行情服务器": "",
        "行情端口": 0,
        "行情授权码": "",
        "交易账号": "",
        "交易密码": "",
        "交易服务器": "",
        "交易端口": 0,
        "交易授权码": "",
        "子账号": "",
        "区域代码": "CN"
    }

    exchanges: List[str] = list(EXCHANGE_VT2TAP.keys())

    def __init__(self, event_engine: EventEngine, gateway_name: str):
        """构造函数"""
        super().__init__(event_engine, gateway_name)

        self.md_api: "QuoteApi" = QuoteApi(self)
        self.td_api: "TradeApi" = TradeApi(self)

    def connect(self, setting: dict) -> None:
        """连接交易接口"""
        quote_username: str = setting["行情账号"]
        quote_password: str = setting["行情密码"]
        quote_host: str = setting["行情服务器"]
        quote_port: int = setting["行情端口"]
        md_authcode: str = setting["行情授权码"]
        trade_username: str = setting["交易账号"]
        trade_password: str = setting["交易密码"]
        trade_host: str = setting["交易服务器"]
        trade_port: int = setting["交易端口"]
        td_authcode: str = setting["交易授权码"]
        client_id: str = setting["子账号"]
        client_location: str = setting["区域代码"]

        if quote_host:
            self.md_api.connect(
                quote_username,
                quote_password,
                quote_host,
                quote_port,
                md_authcode
            )

        if trade_host:
            self.td_api.connect(
                trade_username,
                trade_password,
                trade_host,
                trade_port,
                td_authcode,
                client_id,
                client_location
            )

    def close(self) -> None:
        """关闭接口"""
        self.md_api.close()
        self.td_api.close()

    def subscribe(self, req: SubscribeRequest) -> None:
        """订阅行情"""
        self.md_api.subscribe(req)

    def send_order(self, req: OrderRequest) -> str:
        """委托下单"""
        return self.td_api.send_order(req)

    def cancel_order(self, req: CancelRequest) -> None:
        """委托撤单"""
        self.td_api.cancel_order(req)

    def query_account(self) -> None:
        """查询资金"""
        pass

    def query_position(self) -> None:
        """查询持仓"""
        pass


class QuoteApi(MdApi):
    """行情API"""

    def __init__(self, gateway: TapGateway) -> None:
        """构造函数"""
        super().__init__()

        self.gateway: TapGateway = gateway
        self.gateway_name: str = gateway.gateway_name

        self.connect_status: bool = False

    def onRspLogin(self, error: int, data: dict) -> None:
        """用户登陆请求回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log(f"行情服务器登录失败：{error}")
        else:
            self.connect_status = True
            self.gateway.write_log("行情服务器登录成功")

    def onAPIReady(self) -> None:
        """API状态通知回报"""
        pass

    def onDisconnect(self, reason: int) -> None:
        """服务器连接断开回报"""
        self.connect_status = False
        self.gateway.write_log(f"行情服务器连接断开，原因：{reason}")

    def onRspSubscribeQuote(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """订阅行情回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log(f"订阅行情失败：{error}")
        else:
            self.update_tick(data)

    def onRtnQuote(self, data: dict) -> None:
        """行情数据推送"""
        self.update_tick(data)

    def update_tick(self, data: dict) -> None:
        """切片数据类型转换"""
        symbol: str = data["CommodityNo"] + data["ContractNo1"]
        exchange: Exchange = EXCHANGE_TAP2VT[data["ExchangeNo"]]

        contract_info: ContractInfo = contract_infos.get((symbol, exchange), None)
        if not contract_info:
            self.gateway.write_log(f"行情合约信息无法匹配：{symbol}和{exchange}")
            return

        tick: TickData = TickData(
            symbol=symbol,
            exchange=exchange,
            datetime=generate_datetime(data["DateTimeStamp"]),
            name=contract_info.name,
            volume=data["QTotalQty"],
            last_price=data["QLastPrice"],
            last_volume=data["QLastQty"],
            limit_up=data["QLimitUpPrice"],
            limit_down=data["QLimitDownPrice"],
            open_price=data["QOpeningPrice"],
            high_price=data["QHighPrice"],
            low_price=data["QLowPrice"],
            pre_close=data["QPreClosingPrice"],
            bid_price_1=data["QBidPrice"][0],
            bid_price_2=data["QBidPrice"][1],
            bid_price_3=data["QBidPrice"][2],
            bid_price_4=data["QBidPrice"][3],
            bid_price_5=data["QBidPrice"][4],
            ask_price_1=data["QAskPrice"][0],
            ask_price_2=data["QAskPrice"][1],
            ask_price_3=data["QAskPrice"][2],
            ask_price_4=data["QAskPrice"][3],
            ask_price_5=data["QAskPrice"][4],
            bid_volume_1=data["QBidQty"][0],
            bid_volume_2=data["QBidQty"][1],
            bid_volume_3=data["QBidQty"][2],
            bid_volume_4=data["QBidQty"][3],
            bid_volume_5=data["QBidQty"][4],
            ask_volume_1=data["QAskQty"][0],
            ask_volume_2=data["QAskQty"][1],
            ask_volume_3=data["QAskQty"][2],
            ask_volume_4=data["QAskQty"][3],
            ask_volume_5=data["QAskQty"][4],
            gateway_name=self.gateway_name,
        )
        self.gateway.on_tick(tick)

    def connect(
        self,
        username: str,
        password: str,
        host: str,
        port: int,
        auth_code: str
    ) -> None:
        """连接服务器"""
        # 禁止重复发起连接，会导致异常崩溃
        if self.connect_status:
            return

        self.init()

        # API基本设置
        path: Path = get_folder_path(self.gateway_name.lower())
        self.setTapQuoteAPIDataPath(str(path).encode("GBK"))
        self.setTapQuoteAPILogLevel(APILOGLEVEL_NONE)

        # 创建API
        req: dict = {
            "AuthCode": auth_code,
            "KeyOperationLogPath": str(path).encode("GBK")
        }
        self.createTapQuoteAPI(req, 0)

        # 设置服务器地址
        self.setHostAddress(host, port)

        # 登录账号
        data: dict = {
            "UserNo": username,
            "Password": password,
            "ISModifyPassword": APIYNFLAG_NO,
            "ISDDA": APIYNFLAG_NO
        }
        self.login(data)

    def subscribe(self, req: SubscribeRequest):
        """订阅行情"""
        contract_info: ContractInfo = contract_infos.get((req.symbol, req.exchange), None)
        if not contract_info:
            self.gateway.write_log(
                f"找不到匹配的合约：{req.symbol}和{req.exchange.value}")
            return

        tap_contract: dict = {
            "ExchangeNo": EXCHANGE_VT2TAP[req.exchange],
            "CommodityType": contract_info.commodity_type,
            "CommodityNo": contract_info.commodity_no,
            "ContractNo1": contract_info.contract_no,
            "CallOrPutFlag2": TAPI_CALLPUT_FLAG_NONE
        }

        if contract_info.commodity_type == "O":
            option_contract: ContractData = option_contract_map[req.symbol]

            tap_contract["StrikePrice1"] = option_contract.option_index
            tap_contract["CallOrPutFlag1"] = OPTIONTYPE_VT2TAP.get(
                option_contract.option_type,
                TAPI_CALLPUT_FLAG_NONE
            )
        else:
            tap_contract["CallOrPutFlag1"] = TAPI_CALLPUT_FLAG_NONE

        self.subscribeQuote(tap_contract)

    def close(self):
        """关闭连接"""
        if self.connect_status:
            self.disconnect()
            self.exit()


class TradeApi(TdApi):
    """交易API"""

    def __init__(self, gateway: TapGateway) -> None:
        """构造函数"""
        super().__init__()

        self.gateway: TapGateway = gateway
        self.gateway_name: str = gateway.gateway_name

        self.connect_status: bool = False
        self.account_no: str = ""           # 委托下单时使用
        self.client_id: str = ""            # 子账号，没有可不填
        self.client_location: str = ""      # 投资者所处国家区域

        self.byte_client_id: bytes = b""    # bytes类型
        self.byte_client_location: bytes = b""

        self.cancel_reqs: Dict[str, CancelRequest] = {}       # 撤单请求缓存

        self.sys_local_map: Dict[str, str] = {}
        self.local_sys_map: Dict[str, str] = {}
        self.sys_server_map: Dict[str, str] = {}

        self.init_query: bool = True        # 初始化是否查询日内委托和成交

    def onConnect(self, address: str) -> None:
        """服务器连接成功回报"""
        self.connect_status = True
        self.gateway.write_log("交易服务器连接成功")

    def onRspLogin(self, error: int, data: dict) -> None:
        """用户登陆请求回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log(f"交易服务器登录失败，错误码：{error}")
        else:
            self.gateway.write_log("交易服务器登录成功")

    def onAPIReady(self, code: int) -> None:
        """API状态通知回报"""
        self.qryCommodity()

    def onRspQryCommodity(
        self,
        session: int,
        error: int,
        last: str,
        data: dict,
    ) -> None:
        """交易品种查询回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log("查询交易品种信息失败")
            return

        commodity_info: CommodityInfo = CommodityInfo(
            name=data["CommodityEngName"],
            size=int(data["ContractSize"]),
            pricetick=data["CommodityTickSize"]
        )
        key: tuple = (data["ExchangeNo"], data["CommodityNo"], data["CommodityType"])
        commodity_infos[key] = commodity_info

        if last == "Y":
            self.gateway.write_log("查询交易品种信息成功")
            self.qryContract({})

    def onRspQryContract(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """交易合约查询回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log("查询交易合约信息失败")
            return

        exchange: Exchange = EXCHANGE_TAP2VT.get(data["ExchangeNo"], None)
        key: tuple = (data["ExchangeNo"], data["CommodityNo"], data["CommodityType"])
        commodity_info: CommodityInfo = commodity_infos.get(key, None)

        if not data or not commodity_info:
            return

        product: Product = Product_TAP2VT.get(data["CommodityType"], None)

        if product and exchange:
            # 生成合约代码
            if product == Product.FUTURES:
                symbol: str = data["CommodityNo"] + data["ContractNo1"]
            else:
                symbol: str = data["CommodityNo"] + data["ContractNo1"] + data["CallOrPutFlag1"] + data["StrikePrice1"]

            # 获取合约名称
            if data["ContractName"]:
                name = data["ContractName"]
            elif commodity_info.name:
                name: str = f"{commodity_info.name} {data['ContractNo1']}"
            else:
                name: str = symbol

            # 创建合约对象
            contract: ContractData = ContractData(
                symbol=symbol,
                exchange=exchange,
                name=name,
                product=product,
                size=commodity_info.size,
                pricetick=commodity_info.pricetick,
                net_position=True,
                gateway_name=self.gateway.gateway_name
            )

            # 期权字段处理
            if product == Product.OPTION:
                contract.option_portfolio = data["CommodityNo"] + "_O"
                contract.option_type = OPTIONTYPE_TAP2VT.get(data["CallOrPutFlag1"], None)
                contract.option_strike = float(data["StrikePrice1"])
                contract.option_index = data["StrikePrice1"]
                contract.option_expiry = datetime.strptime(data["ContractExpDate"], "%Y-%m-%d")
                contract.option_underlying = data["CommodityNo"] + data["ContractNo1"]

                option_contract_map[symbol] = contract

            # 推送合约对象
            self.gateway.on_contract(contract)

            # 合约数据缓存
            contract_info: ContractInfo = ContractInfo(
                name=contract.name,
                exchange_no=data["ExchangeNo"],
                contract_no=data["ContractNo1"],
                commodity_type=data["CommodityType"],
                commodity_no=data["CommodityNo"],
            )
            contract_infos[(contract.symbol, contract.exchange)] = contract_info

        if last == "Y":
            self.gateway.write_log("查询交易合约信息成功")
            self.query_account()

    def onRspQryAccount(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """账户查询回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log("查询账号信息失败")
            return

        req: dict = {
            "AccountNo": data["AccountNo"]
        }
        self.qryFund(req)

    def onRspQryFund(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """账户资金查询回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log("查询资金信息失败")
            return

        self.update_account(data)

        if last == "Y":
            self.gateway.write_log("查询资金信息成功")
            self.query_position()

    def onRtnFund(self, data: dict) -> None:
        """账户资金推送"""
        self.update_account(data)

    def onRspQryPositionSummary(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """持仓汇总查询回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log("查询持仓信息失败")
            return

        if data:
            self.update_position(data)

        if last == "Y":
            self.gateway.write_log("查询持仓信息成功")

            if self.init_query:
                self.query_order()

    def onRtnPositionSummary(self, data: dict) -> None:
        """持仓汇总更新推送"""
        self.update_position(data)

    def onRspQryOrder(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """当日委托查询回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log("查询委托信息失败")
            return

        if data:
            self.update_order(data)

        if last == "Y":
            self.gateway.write_log("查询委托信息成功")

            if self.init_query:
                self.query_trade()

    def onRtnOrder(self, data: dict) -> None:
        """委托查询推送"""
        if data["ErrorCode"] != TAPIERROR_SUCCEED:
            self.gateway.write_log(f"委托下单失败，错误码: {data['ErrorCode']}")
            return

        self.update_order(data)

    def onRspQryFill(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """当日成交查询回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log("查询成交信息失败")
            return

        if data:
            self.update_trade(data)

        if last == "Y":
            self.gateway.write_log("查询成交信息成功")

    def onRtnFill(self, data: dict) -> None:
        """成交更新推送"""
        self.update_trade(data)

    def onRspOrderAction(
        self,
        session: int,
        error: int,
        data: dict
    ) -> None:
        """撤单和修改委托回报"""
        if error != TAPIERROR_SUCCEED:
            self.gateway.write_log(f"委托操作失败：{error}")
            return

    def update_account(self, data: dict) -> None:
        """更新并推送资金"""
        self.account_no: str = data["AccountNo"]

        account: AccountData = AccountData(
            accountid=data["AccountNo"],
            balance=data["Balance"],
            frozen=data["Balance"] - data["Available"],
            gateway_name=self.gateway_name
        )
        self.gateway.on_account(account)

    def update_position(self, data: dict) -> None:
        """更新并推送持仓"""
        position: PositionData = PositionData(
            symbol=data["CommodityNo"] + data["ContractNo"],
            exchange=EXCHANGE_TAP2VT.get(data["ExchangeNo"], None),
            direction=DIRECTION_TAP2VT[data["MatchSide"]],
            volume=data["PositionQty"],
            price=data["PositionPrice"],
            gateway_name=self.gateway_name
        )
        self.gateway.on_position(position)

    def update_order(self, data: dict) -> None:
        """更新并推送委托"""
        # 过滤撤销和修改状态中的委托
        if data["OrderState"] in {"7", "8"}:
            return

        self.local_sys_map[data["ClientOrderNo"]] = data["OrderNo"]
        self.sys_local_map[data["OrderNo"]] = data["ClientOrderNo"]
        self.sys_server_map[data["OrderNo"]] = data["ServerFlag"]

        order: OrderData = OrderData(
            symbol=data["CommodityNo"] + data["ContractNo"],
            exchange=EXCHANGE_TAP2VT.get(data["ExchangeNo"], None),
            orderid=data["ClientOrderNo"],
            type=ORDERTYPE_TAP2VT.get(data["OrderType"], data["OrderType"]),
            direction=DIRECTION_TAP2VT[data["OrderSide"]],
            price=data["OrderPrice"],
            volume=data["OrderQty"],
            traded=data["OrderMatchQty"],
            status=STATUS_TAP2VT.get(data["OrderState"], Status.SUBMITTING),
            datetime=generate_datetime(data["OrderInsertTime"]),
            gateway_name=self.gateway_name
        )
        self.gateway.on_order(order)

        # 发送等待撤单请求
        if data["ClientOrderNo"] in self.cancel_reqs:
            req: str = self.cancel_reqs.pop(data["ClientOrderNo"])
            self.cancel_order(req)

    def update_trade(self, data: dict) -> None:
        """更新并推送成交"""
        orderid: str = self.sys_local_map[data["OrderNo"]]

        trade: TradeData = TradeData(
            symbol=data["CommodityNo"] + data["ContractNo"],
            exchange=EXCHANGE_TAP2VT.get(data["ExchangeNo"], None),
            orderid=orderid,
            tradeid=data["MatchNo"],
            direction=DIRECTION_TAP2VT[data["MatchSide"]],
            price=data["MatchPrice"],
            volume=data["MatchQty"],
            datetime=generate_datetime(data["MatchDateTime"]),
            gateway_name=self.gateway_name
        )
        self.gateway.on_trade(trade)

    def connect(
        self,
        username: str,
        password: str,
        host: str,
        port: int,
        auth_code: str,
        client_id: str,
        client_location: str,
        init_query: bool = True
    ) -> None:
        """连接服务器"""
        # 禁止重复发起连接，会导致异常崩溃
        if self.connect_status:
            return

        self.client_id = client_id
        self.client_location = client_location
        self.init_query = init_query

        self.byte_client_id = client_id.encode()
        self.byte_client_location = client_location.encode()

        self.init()

        # API基本设置
        path: Path = get_folder_path(self.gateway_name.lower())

        # 创建API
        req: dict = {
            "AuthCode": auth_code,
            "KeyOperationLogPath": str(path).encode("GBK"),
            "LogLevel": APILOGLEVEL_NONE
        }
        self.createITapTradeAPI(req, 0)

        # 设置服务器地址
        self.setHostAddress(host, port)

        # 登陆
        data: dict = {
            "UserNo": username,
            "Password": password,
            "ISModifyPassword": APIYNFLAG_NO,
            "NoticeIgnoreFlag": "TAPI_NOTICE_IGNORE_POSITIONPROFIT"
        }
        self.login(data)

    def send_order(self, req: OrderRequest) -> str:
        """委托下单"""
        contract_info: ContractInfo = contract_infos.get((req.symbol, req.exchange), None)
        if not contract_info:
            self.gateway.write_log(f"找不到匹配的合约：{req.symbol}和{req.exchange.value}")
            return ""

        if req.type not in ORDERTYPE_VT2TAP:
            self.gateway.write_log(f"不支持的委托类型: {req.type.value}")
            return ""

        order_req: dict = {
            "AccountNo": self.account_no,
            "ExchangeNo": contract_info.exchange_no,
            "CommodityType": contract_info.commodity_type,
            "CommodityNo": contract_info.commodity_no,
            "ContractNo": contract_info.contract_no,
            "OrderType": ORDERTYPE_VT2TAP[req.type],
            "OrderSide": DIRECTION_VT2TAP[req.direction],
            "OrderPrice": req.price,
            "OrderQty": int(req.volume),
        }

        if self.client_id:
            order_req["ClientID"] = self.client_id
            order_req["ClientLocationID"] = self.client_location

        if contract_info.commodity_type == "O":
            option_contract: ContractData = option_contract_map[req.symbol]

            order_req["StrikePrice"] = option_contract.option_index
            order_req["CallOrPutFlag"] = OPTIONTYPE_VT2TAP.get(
                option_contract.option_type,
                TAPI_CALLPUT_FLAG_NONE
            )

        # byte_order_id是bytes类型数据
        error_id, session, byte_order_id = self.insertOrder(order_req)

        if self.byte_client_id in byte_order_id:
            prefix: bytes = b"#" + self.byte_client_id + b"#" + self.byte_client_location + b"#"
            byte_order_id = byte_order_id.replace(prefix, "")

        order_id = byte_order_id.decode()

        order: OrderData = req.create_order_data(
            order_id,
            self.gateway_name
        )

        if error_id:
            self.gateway.write_log(f"委托请求失败，错误号：{error_id}")
            order.status = Status.REJECTED

        self.gateway.on_order(order)

        return order.vt_orderid

    def cancel_order(self, req: CancelRequest) -> None:
        """委托撤单"""
        order_no: str = self.local_sys_map.get(req.orderid, "")
        if not order_no:
            self.cancel_reqs[req.orderid] = req
            return

        server_flag: str = self.sys_server_map[order_no]

        cancel_req: dict = {
            "OrderNo": order_no,
            "ServerFlag": server_flag,
        }
        self.cancelOrder(cancel_req)

    def query_account(self) -> None:
        """资金查询"""
        self.qryAccount({})

    def query_position(self) -> None:
        """持仓查询"""
        self.qryPositionSummary({})

    def query_order(self) -> None:
        """当日委托查询"""
        self.qryOrder({})

    def query_trade(self) -> None:
        """当日成交查询"""
        self.qryFill({})

    def close(self):
        """关闭连接"""
        if self.connect_status:
            self.disconnect()
            self.exit()


def generate_datetime(timestamp: str) -> datetime:
    """生成datetime格式时间"""
    if "-" in timestamp:
        if "." in timestamp:
            dt: datetime = datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S.%f")
        else:
            dt: datetime = datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S")
    else:
        dt: datetime = datetime.strptime(timestamp, "%y%m%d%H%M%S.%f")

    dt: datetime = dt.replace(tzinfo=CHINA_TZ)
    return dt


@dataclass
class ContractInfo:
    """存储合约信息"""
    name: str
    exchange_no: str
    commodity_type: int
    commodity_no: str
    contract_no: str


@dataclass
class CommodityInfo:
    """存储品种信息"""
    name: str
    size: int
    pricetick: float



import platform


from setuptools import Extension, setup


def get_ext_modules() -> list:
    """
    获取三方模块
    Windows需要编译封装接口
    Linux和Mac由于缺乏二进制库支持无法使用
    """
    # Linux
    if platform.system() == "Linux":
        include_dirs = ["vnpy_tap/api/include", "vnpy_tap/api/vntap"]
        library_dirs = ["vnpy_tap/api"]
        extra_compile_flags = [
            "-std=c++17",
            "-O3",
            "-Wno-delete-incomplete",
            "-Wno-sign-compare",
        ]
        extra_link_args = ["-lstdc++"]
        runtime_library_dirs = ["$ORIGIN"]
        libraries = ["iTapTradeAPI", "TapQuoteAPI"]
    # Windows
    elif platform.system() == "Windows":
        include_dirs = ["vnpy_tap/api/include", "vnpy_tap/api/vntap"]
        library_dirs = ["vnpy_tap/api/libs", "vnpy_tap/api"]
        extra_compile_flags = ["-O2", "-MT"]
        extra_link_args = []
        runtime_library_dirs = []
        libraries = ["iTapTradeAPI_64", "TapQuoteAPI"]

    vntapmd = Extension(
        "vnpy_tap.api.vntapmd",
        [
            "vnpy_tap/api/vntap/vntapmd/vntapmd.cpp",
        ],
        include_dirs=include_dirs,
        define_macros=[],
        undef_macros=[],
        library_dirs=library_dirs,
        libraries=libraries,
        extra_compile_args=extra_compile_flags,
        extra_link_args=extra_link_args,
        runtime_library_dirs=runtime_library_dirs,
        depends=[],
        language="cpp",
    )

    vntaptd = Extension(
        "vnpy_tap.api.vntaptd",
        [
            "vnpy_tap/api/vntap/vntaptd/vntaptd.cpp",
        ],
        include_dirs=include_dirs,
        define_macros=[],
        undef_macros=[],
        library_dirs=library_dirs,
        libraries=libraries,
        extra_compile_args=extra_compile_flags,
        extra_link_args=extra_link_args,
        runtime_library_dirs=runtime_library_dirs,
        depends=[],
        language="cpp",
    )

    return [vntaptd, vntapmd]


setup(
    ext_modules=get_ext_modules(),
)



APIYNFLAG_YES = "Y"
APIYNFLAG_NO = "N"
APILOGLEVEL_NONE = "N"
APILOGLEVEL_ERROR = "E"
APILOGLEVEL_WARNING = "W"
APILOGLEVEL_DEBUG = "D"
TAPI_COMMODITY_TYPE_NONE = "N"
TAPI_COMMODITY_TYPE_SPOT = "P"
TAPI_COMMODITY_TYPE_FUTURES = "F"
TAPI_COMMODITY_TYPE_OPTION = "O"
TAPI_COMMODITY_TYPE_SPREAD_MONTH = "S"
TAPI_COMMODITY_TYPE_SPREAD_COMMODITY = "M"
TAPI_COMMODITY_TYPE_BUL = "U"
TAPI_COMMODITY_TYPE_BER = "E"
TAPI_COMMODITY_TYPE_STD = "D"
TAPI_COMMODITY_TYPE_STG = "G"
TAPI_COMMODITY_TYPE_PRT = "R"
TAPI_COMMODITY_TYPE_BLT = "L"
TAPI_COMMODITY_TYPE_BRT = "Q"
TAPI_COMMODITY_TYPE_DIRECTFOREX = "X"
TAPI_COMMODITY_TYPE_INDIRECTFOREX = "I"
TAPI_COMMODITY_TYPE_CROSSFOREX = "C"
TAPI_COMMODITY_TYPE_INDEX = "Z"
TAPI_COMMODITY_TYPE_STOCK = "T"
TAPI_COMMODITY_TYPE_SPOT_TRADINGDEFER = "Y"
TAPI_COMMODITY_TYPE_FUTURE_LOCK = "J"
TAPI_COMMODITY_TYPE_EFP = "A"
TAPI_COMMODITY_TYPE_TAS = "B"
TAPI_CALLPUT_FLAG_CALL = "C"
TAPI_CALLPUT_FLAG_PUT = "P"
TAPI_CALLPUT_FLAG_NONE = "N"
TAPI_AUTHTYPE_DIRECT = "1"
TAPI_AUTHTYPE_RELAY = "2"

TAPIERROR_SUCCEED = 0
TAPIERROR_ConnectFail = -1
TAPIERROR_LinkAuthFail = -2
TAPIERROR_HostUnavailable = -3
TAPIERROR_SendDataError = -4
TAPIERROR_TestIDError = -5
TAPIERROR_NotReadyTestNetwork = -6
TAPIERROR_CurTestNotOver = -7
TAPIERROR_NOFrontAvailable = -8
TAPIERROR_DataPathAvaiable = -9
TAPIERROR_RepeatLogin = -10
TAPIERROR_InnerError = -11
TAPIERROR_LastReqNotFinish = -12
TAPIERROR_InputValueError = -13
TAPIERROR_AuthCode_Invalid = -14
TAPIERROR_AuthCode_Expired = -15
TAPIERROR_AuthCode_TypeNotMatch = -16
TAPIERROR_API_NotReady = -17
TAPIERROR_UDP_LISTEN_FAILED = -18
TAPIERROR_UDP_LISTENING = -19
TAPIERROR_NotImplemented = -20
TAPIERROR_CallOneTimeOnly = -21
TAPIERROR_Frequently = -22
TAPIERROR_DataCollect = -23
TAPIERROR_DataLoad = -24
TAPIERROR_NoRelay = -25
TAPIERROR_INPUTERROR_NULL = -10000
TAPIERROR_INPUTERROR_TAPIYNFLAG = -10001
TAPIERROR_INPUTERROR_TAPILOGLEVEL = -10002
TAPIERROR_INPUTERROR_TAPICommodityType = -10003
TAPIERROR_INPUTERROR_TAPICallOrPutFlagType = -10004
TAPIERROR_INPUTERROR_TAPIAccountType = -12001
TAPIERROR_INPUTERROR_TAPIAccountState = -12003
TAPIERROR_INPUTERROR_TAPIAccountFamilyType = -12004
TAPIERROR_INPUTERROR_TAPIOrderTypeType = -12005
TAPIERROR_INPUTERROR_TAPIOrderSourceType = -12006
TAPIERROR_INPUTERROR_TAPITimeInForceType = -12007
TAPIERROR_INPUTERROR_TAPISideType = -12008
TAPIERROR_INPUTERROR_TAPIPositionEffectType = -12009
TAPIERROR_INPUTERROR_TAPIHedgeFlagType = -12010
TAPIERROR_INPUTERROR_TAPIOrderStateType = -12011
TAPIERROR_INPUTERROR_TAPICalculateModeType = -12012
TAPIERROR_INPUTERROR_TAPIMatchSourceType = -12013
TAPIERROR_INPUTERROR_TAPIOpenCloseModeType = -12014
TAPIERROR_INPUTERROR_TAPIFutureAlgType = -12015
TAPIERROR_INPUTERROR_TAPIOptionAlgType = -12016
TAPIERROR_INPUTERROR_TAPIBankAccountLWFlagType = -12017
TAPIERROR_INPUTERROR_TAPIMarginCalculateModeType = -12021
TAPIERROR_INPUTERROR_TAPIOptionMarginCalculateModeType = -12022
TAPIERROR_INPUTERROR_TAPICmbDirectType = -12023
TAPIERROR_INPUTERROR_TAPIDeliveryModeType = -12024
TAPIERROR_INPUTERROR_TAPIContractTypeType = -12025
TAPIERROR_INPUTERROR_TAPITacticsTypeType = -12035
TAPIERROR_INPUTERROR_TAPIORDERACT = -12036
TAPIERROR_INPUTERROR_TAPITriggerConditionType = -12041
TAPIERROR_INPUTERROR_TAPITriggerPriceTypeType = -12042
TAPIERROR_INPUTERROR_TAPITradingStateType = -12043
TAPIERROR_INPUTERROR_TAPIMarketLevelType = -12044
TAPIERROR_INPUTERROR_TAPIOrderQryTypeType = -12045
TAPIERROR_DISCONNECT_CLOSE_INIT = 1
TAPIERROR_DISCONNECT_CLOSE_PASS = 2
TAPIERROR_DISCONNECT_READ_ERROR = 3
TAPIERROR_DISCONNECT_WRITE_ERROR = 4
TAPIERROR_DISCONNECT_BUF_FULL = 5
TAPIERROR_DISCONNECT_IOCP_ERROR = 6
TAPIERROR_DISCONNECT_PARSE_ERROR = 7
TAPIERROR_DISCONNECT_CONNECT_TIMEOUT = 8
TAPIERROR_DISCONNECT_INIT_ERROR = 9
TAPIERROR_DISCONNECT_HAS_CONNECTED = 10
TAPIERROR_DISCONNECT_HAS_EXIT = 11
TAPIERROR_DISCONNECT_TRY_LATER = 12
TAPIERROR_DISCONNECT_HEARTBEAT_FAILED = 13
TAPIERROR_LOGIN = 10001
TAPIERROR_LOGIN_USER = 10002
TAPIERROR_LOGIN_DDA = 10003
TAPIERROR_LOGIN_LICENSE = 10004
TAPIERROR_LOGIN_MODULE = 10005
TAPIERROR_LOGIN_FORCE = 10006
TAPIERROR_LOGIN_STATE = 10007
TAPIERROR_LOGIN_PASS = 10008
TAPIERROR_LOGIN_RIGHT = 10009
TAPIERROR_LOGIN_COUNT = 10010
TAPIERROR_LOGIN_NOTIN_SERVERFLAGUSRES = 10011
TAPIERROR_LOGIN_USER_EXPIRED = 10012
TAPIERROR_LOGIN_NO_ACCOUNT = 10013
TAPIERROR_LOGIN_NO_JGT = 10014
TAPIERROR_LOGIN_ERROR_TIMES = 10015
TAPIERROR_LOGIN_ERROR_AUTHTYPE = 10016
TAPIERROR_LOGIN_ERROR_AUTHEXPIRED = 10017
TAPIERROR_LOGIN_ERROR_PASSWDEXPIRED = 10018
TAPIERROR_LOGIN_ERROR_USERTRUST = 10019
TAPIERROR_LOGIN_ERROR_CLIENTVERSION = 10020
TAPIERROR_LOGIN_ERROR_AUTHCODEINFO = 10021
TAPIERROR_USERINFO_QRY = 10101
TAPIERROR_USERALLRIGHT_QRY = 11001
TAPIERROR_USERALLACCOUNT_QRY = 11501
TAPIERROR_USERPASSWORD_MOD = 11701
TAPIERROR_USERPASSWORD_MOD_SOURCE = 11702
TAPIERROR_USERPASSWORD_MOD_SAME = 11703
TAPIERROR_USERPASSWORD_MOD_COMPLEXITY = 11704
TAPIERROR_ACCOUNTINFO_QRY = 20201
TAPIERROR_TRADENO_QRY = 20701
TAPIERROR_CONTRACTINFO_QRY = 22801
TAPIERROR_SPECIALOPTIONFUTURE_QRY = 22901
TAPIERROR_COMMODITYORDERTYPE_QRY = 25501
TAPIERROR_ORDERTIMEINFORCE_QRY = 25601
TAPIERROR_USER_ORDER_FREQUENCE_QRY = 28901
TAPIERROR_USERSUBMITAUTHTYPE_ERROR = 29591
TAPIERROR_USERSUBMITINFO_EMPTY = 29592
TAPIERROR_USERAUTHKEYVERSION_ERROR = 29593
TAPIERROR_USERSUBMITINFO_PARTY = 29594
TAPIERROR_USERSUBMITINFO_TESTKEY = 29595
TAPIERROR_USERSUBMITINFO_USERNO = 29596
TAPIERROR_ORDERINSERT_ACCOUNT = 60001
TAPIERROR_ORDERINSERT_ACCOUNT_STATE = 60002
TAPIERROR_ORDERINSERT_SIDE_TRADE = 60003
TAPIERROR_ORDERINSERT_OPTIONS_TRADE = 60004
TAPIERROR_ORDERINSERT_COMMODITY_TRADE = 60005
TAPIERROR_ORDERINSERT_OPEN_RIGHT = 60006
TAPIERROR_ORDERINSERT_RISK_CHECK = 60007
TAPIERROR_ORDERINSERT_CONTRACT = 60011
TAPIERROR_ORDERINSERT_TRADEROUTE = 60021
TAPIERROR_ORDERINSERT_POSITIONMAX = 60022
TAPIERROR_ORDER_NOTRADE = 60023
TAPIERROR_ORDER_CLOSE = 60024
TAPIERROR_ORDERINSERT_NOTENOUGHFUND = 60031
TAPIERROR_ORDERINSERT_ORDERTYPE = 60032
TAPIERROR_ORDERINSERT_TIMEINFORCE = 60033
TAPIERROR_ORDERINSERT_NO_TACTICS = 60034
TAPIERROR_ORDERINSERT_POSITION_CANNOT_CLOSE = 60035
TAPIERROR_ORDERINSERT_AUTOCHECK_FAIL = 60036
TAPIERROR_ORDERINSERT_LME_NOTREADY = 60037
TAPIERROR_ORDERINSERT_CLOSEMODE = 60038
TAPIERROR_ORDERINSERT_PARENTNOTENOUGHFUND = 60039
TAPIERROR_SWAP_CONTRACT = 60040
TAPIERROR_ORDERINSERT_PRICE = 60041
TAPIERROR_ORDERINSERT_EFFECT = 60042
TAPIERROR_ORDERINSERT_TARGETCONTRACT = 60043
TAPIERROR_USERNO_NOTHAS_ACCOUNT = 60051
TAPIERROR_UPPERCHANNEL_BROKEN = 60052
TAPIERROR_UPPERCHANNEL_NOT_EXIST = 60053
TAPIERROR_ORDERDELETE_NOT_SYSNO = 60061
TAPIERROR_ORDERDELETE_NOT_STATE = 60062
TAPIERROR_ORDERACTIVE_NOT_STATE = 60063
TAPIERROR_ORDERDELETE_NOT_LAST = 60064
TAPIERROR_ORDERCHECK_NOT_STATE = 60071
TAPIERROR_ORDERCHECK_FAIL = 60072
TAPIERROR_ORDERMODIFY_NOT_STATE = 60081
TAPIERROR_ORDERMODIFY_BACK_INPUT = 60082
TAPIERROR_ORDERINSERT_FEE = 60091
TAPIERROR_ORDERINSERT_MARGIN = 60092
TAPIERROR_ORDER_NO_PERMIT = 60100
TAPIERROR_RSPQUOTE_NO_PERMIT = 60101
TAPIERROR_RSPQUOTE_CHILD_NO_PERMIT = 60102
TAPIERROR_TRADENO_NOT_FIND = 60103
TAPIERROR_ORDER_NO_CLOSE = 60104
TAPIERROR_QRY_QUOTE_NO_PERMIT = 60105
TAPIERROR_EXPIREDATE_NO_PERMIT = 60106
TAPIERROR_CMB_NO_PERMIT = 60107
TAPIERROR_ORDERSERVER_NO_PERMIT = 60108
TAPIERROR_POSITION_CANNOT_EXEC_OR_ABANDON = 60109
TAPIERROR_ORDERCHECK_NO_PERMIT = 60110
TAPIERROR_ORDERMAXOPENVOL_NO_PERMIT = 60111
TAPIERROR_ORDERLIMIT_OPEN_NO_PERMIT = 60112
TAPIERROR_ORDER_SINGLEVOL_NO_PERMIT = 60113
TAPIERROR_ORDER_POSITIONVOL_NO_PERMIT = 60114
TAPIERROR_ORDER_QTY_NO_PERMIT = 60115
TAPIERROR_ORDER_APPLY_NO_PERMIT = 60117
TAPIERROR_ORDER_FREQ_OVERRUN = 60118
TAPIERROR_COMB_NO_SIDEORHEDGE = 60119
TAPIERROR_REQQUOTE_EXITREASONABLPRICE = 60120
TAPIERROR_RSPQUOTE_PRICE = 60121
TAPIERROR_RISKORDER_CANCEL = 60122
TAPIERROR_RSPQUOTE_EFFECT = 60123
TAPIERROR_ORDERINSERT_SIDEMODE = 60124
TAPIERROR_AUTOORDER_MAXCOUNT = 60125
TAPIERROR_SHFEDEEPQUOTE_LIMIT = 60126
TAPIERROR_SHFEDEEPQUOTE_NODATA = 60127
TAPIERROR_SELFMATCH = 60128
TAPIERROR_ERRORORDER_MAXCOUNT = 60129
TAPIERROR_ORDER_UPPERFREQ_OVERRUN = 60130
TAPIERROR_ORDER_FREQUENCY = 61001
TAPIERROR_ORDER_QUERYING = 61002
TAPIERROR_SUBSCRIBEQUOTE_MAX = 72001
TAPIERROR_SUBSCRIBEQUOTE_EXCHANGE_MAX = 72002
TAPIERROR_SUBSCRIBEQUOTE_NO_RIGHT = 72101
TAPIERROR_SUBSCRIBEQUOTE_NO_EXCHANGE_RIGHT = 72102
TAPIERROR_SUBSCRIBEQUOTE_COMMODITY_NOT_EXIST = 72103
TAPIERROR_SUBSCRIBEQUOTE_CONTRACT_MAY_NOT_EXIST = 72104
TAPIERROR_QUOTEFRONT_UNKNOWN_PROTOCOL = 83001
TAPIERROR_QUOTEFRONT_LOGIN_TOTALCOUNT = 83002
TAPIERROR_QUOTEFRONT_SUB_COUNT = 83003
TAPIERROR_ORDER_SEND = 80001
TAPIERROR_DLG_NULL = 80002
TAPIERROR_ORDER_FIELD = 80003
TAPIERROR_TRADE_REJ_BYUPPER = 80004
TAPIERROR_ORDER_FORBIDEXEC = 80005
